package de.assono;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import lotus.domino.Database;
import lotus.domino.DxlImporter;
import lotus.domino.NotesException;
import lotus.domino.NotesFactory;
import lotus.domino.NotesThread;
import lotus.domino.Session;
import lotus.domino.Stream;




public class ImporterApp {
	
	static final String DB_SERVER = "";
	static final String DB_FILE_PATH = "Projekte\\Konferenzen\\EntwicklerCamp2013\\EC13-T3S2-Java-in-Notes.nsf";
	//static final String DB_SERVER = "Notebook-019-9/TBahn";
	//static final String DB_FILE_PATH = "Projekte\\Konferenzen\\EntwicklerCamp 2013\\EC13-T3S2-Java-in-Notes.nsf";
	
	static final String FILE_SEPARATOR = System.getProperty("file.separator");
	static final String BASE_DIRECTORY = System.getProperty("user.home") + FILE_SEPARATOR + "Desktop" + FILE_SEPARATOR;
	static final String IMPORT_DIRECTORY = BASE_DIRECTORY + "Import";
	static final String IMPORTED_DIRECTORY = BASE_DIRECTORY + "Imported";

	static Session session = null;

	public static void main(String[] args) {

		NotesThread.sinitThread();
		try {
			do {
				if (session == null || !session.isValid()) {
					System.out.println("Initialisiere Session...");
					session = NotesFactory.createSession("");
				}
				monitorDirectory();
				Thread.sleep(5000);
			} while (System.in.available() == 0);
			System.out.println("Beendet");
			session.recycle();
		} catch (NotesException ne) {
			System.out.println("Notes-Fehler #" + ne.id + ": " + ne.text);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			NotesThread.stermThread();
		}
	}

	private static void monitorDirectory() throws NotesException {

		System.out.println("Teste das Import-Verzeichnis(" + IMPORT_DIRECTORY + ")...");
		File importDirectory = new File(IMPORT_DIRECTORY);
		File[] fileList = importDirectory.listFiles();
		for (int i = 0; i < fileList.length; i++) {
			if (fileList[i].isFile() && fileList[i].getName().endsWith(".dxl")) {
				importDXL(fileList[i]);
			}
		}
		if (fileList.length > 0) {
			zipImportedFiles(fileList);
		}
	}

	private static void importDXL(File file) throws NotesException {

		String fileName = file.getName();
		System.out.println("Importiere Datei '" + fileName + "'...");
		Database db = session.getDatabase(DB_SERVER, DB_FILE_PATH);
		if (db == null) {
			throw new RuntimeException("Variable db ist gleich null");
		}
		if (!db.isOpen()) {
			throw new RuntimeException("Datenbank ist nicht offen");
		}
		Stream stream = session.createStream();
		try {
			if (stream.open(file.getCanonicalPath()) & (stream.getBytes() > 0)) {
				DxlImporter importer = session.createDxlImporter();
				importer.setDocumentImportOption(DxlImporter.DXLIMPORTOPTION_CREATE);
				importer.setAclImportOption(DxlImporter.DXLIMPORTOPTION_IGNORE);
				importer.setDesignImportOption(DxlImporter.DXLIMPORTOPTION_IGNORE);
				importer.setInputValidationOption(DxlImporter.DXLVALIDATIONOPTION_VALIDATE_ALWAYS);
				importer.setReplaceDbProperties(false);
				importer.setReplicaRequiredForReplaceOrUpdate(false);
				importer.setUnknownTokenLogOption(DxlImporter.DXLLOGOPTION_WARNING);
				try {
					importer.importDxl(stream, db);
				} catch (NotesException ne) {
					ne.printStackTrace();
					System.out.println(importer.getLog());
				}
				stream.close();
				System.out.println(importer.getImportedNoteCount() + " Dokumente aus der Datei importiert");
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	private static void zipImportedFiles(File[] importedFiles) {

		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss.'zip'");
		String zipfile = IMPORTED_DIRECTORY + File.separator + df.format(new Date());
		byte[] buffer = new byte[4096];
		int bytes_read;
		try {
			ZipOutputStream out = new ZipOutputStream(new FileOutputStream(zipfile));
			for (int i = 0; i < importedFiles.length; ++i) {
				File file = importedFiles[i];
				FileInputStream in = new FileInputStream(file);
				ZipEntry entry = new ZipEntry(file.getName());
				out.putNextEntry(entry);
				while ((bytes_read = in.read(buffer)) != -1) {
					out.write(buffer, 0, bytes_read);
				}
				in.close();
				if (file.delete()) {
					System.out.println("Datei '" + file.getName() + "' entfernt");
				}
			}
			out.close();
		} catch (FileNotFoundException fnfe) {
			fnfe.printStackTrace();
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
}
