package com.mindoo.dde.perspectiveswitcher.actions;

import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.actions.ActionDelegate;

/**
 * Abstract action that opens an Eclipse perspective with a specific ID.
 * 
 * @author Karsten Lehmann
 */
public abstract class AbstractOpenPerspectiveAction extends ActionDelegate implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow m_window;
	private IAction m_action;

	public void init(IWorkbenchWindow window) {
		m_window=window;
	}

	@Override
	public void dispose() {
		super.dispose();

		m_window=null;
		m_action=null;
	}

	@Override
	public void init(IAction action) {
		super.init(action);

		m_action=action;
	}

	public abstract String getPerspectiveId();
	
	@Override
	public void run(IAction action) {
		Display.getDefault().asyncExec(new Runnable() {

			@Override
			public void run() {
				String pId=getPerspectiveId();
				
				//Grab the perspective from the registry
				IPerspectiveDescriptor pDesc=m_window.getWorkbench().getPerspectiveRegistry().findPerspectiveWithId(pId);
				if (pDesc!=null) {
					try {
						//show it
						m_window.getWorkbench().showPerspective(pId, m_window);
					} catch (WorkbenchException e) {
						e.printStackTrace();
					}
				}
				else {
					System.err.println("Perspective with id '"+pId+"' not found");
				}
			}
		});
	}
}
